function [dataSet, setName] = LoadOptoSet(fileName)
%LOADOPTOSET  - load trajectory dataset (OPTOTRAK file format)
%
% 	usage:  [dataSet, setName] = LoadOptoSet(fileName)
%
% where FILENAME specifies an OPTOTRAK data file
% (if empty selected using interactive dialog box)
%
% Expects to find two files named with the convention
%	C_###.ext	(OPTOTRAK data)
%	O1_###.ext	(speech data; loaded if available)
% where ### is the utterance number and ext is an arbitrary extension
% and <###.ext> together form the SETNAME
%
% returns DATASET, an array of structs with fields
%	NAME	- trajectory name
%	SRATE	- sampling rate (Hz)
%	SIGNAL	- signal data [nSamp (samples) x nComp (components)]
%				where recognized components are X Y Z
%
% assigns letters in ascending order as the name of each trajectory found in the C file

% 03/03/00  mark tiede, incorporating T. Kuratate's OPTOREAD.M

%	parse args

if nargin < 1, fileName = []; end;

%	get setName interactively if necessary

if isempty(fileName),
	[file, pathName] = uigetfile('C*', 'Select OPTOTRAK dataset member');
	if file == 0, return; end;
	fileName = [pathName file];
end;

[pathName, file, ext] = fileparts(fileName);
setName = [file(3:end) ext];
trajName = [pathName file ext];
if exist(trajName) ~= 2, error([trajName ' not found']); end;
audioName = [pathName 'O1_' setName];
if exist(audioName) ~= 2, warning([audioName ' not found']); audioName = []; end;

% 	load audio if available

dataSet = struct('NAME', [], 'SRATE', [], 'SIGNAL', []);
if isempty(audioName),
	base = 1;
else,
	base = 2;
	[h,s] = optoread(audioName);
	dataSet.NAME = 'AUDIO';
	dataSet.SRATE = h.frequency;
	dataSet.SIGNAL = s;
end;

% 	load trajectory data

[h,s] = optoread(trajName);
name = double('A');
for n = 0 : h.items-1,
	dataSet(base+n) = struct('NAME', char(name+n), 'SRATE', h.frequency, ...
		'SIGNAL', [s(:,n*3+1) , s(:,n*3+2) , s(:,n*3+3)]);
end;


%=============================================================================
% OPTOREAD  - parse OPTOTRAK data files
%
% 	usage:  [header, data] = optoread(fileName)
%
% returns file header and trajectory data (nSamps x nTraj)
%
% 	Header format:
%
%  Name          | Size, Type  | Description
% ---------------+-------------+----------------------------------------------
% filetype       | 1, 'int8'   | File type identifier (always 32 for floating point files)
% items          | 1, 'int16'  | Number of items in each frame data
% subitems       | 1, 'int16'  | Number of floating point subitems in each frame
% numframes      | 1, 'int32'  | Number of frames of data
% frequency      | 1, 'float32'| Frequency at wich data was collected
% user_comments  | 60, 'char'  | A user-supplied comment
% sys_comments   | 60, 'char'  | Unused at present
% descrip_file   | 30, 'char'  | Unused at present
% cutoff         | 1, 'int16'  | Cutoff frequency used if 3D data was filtered
% coll_time      | 10, 'char'  | The time at which the data was collected (hh-mm-ss)
% coll_date      | 10, 'char'  | The date on which the data was collected (mm-dd-yy)
% frame_start    | 1, 'int32'  | Unused at present
% extended_header| 1, 'int16'  | Contains the value 12345 if the following four fields are also used 
% char_subitems  | 1, 'int16'  | Number of character subitems
% int_subitems   | 1, 'int16'  | Number of integer subitems
% dbl_subitems   | 1, 'int16'  | Number of double precision floating point subitems
% item_size      | 1, 'int16'  | Unused at present
% padding        | 57, 'char'  | Unused at present

function [header, data] = optoread(fileName)

header = struct('filetype', 0, 'items', 0, 'subitems', 0, 'numframes', 0,... 
    'frequency', 0, 'user_comments', [], 'sys_comments', [],...
    'descrip_file', [], 'cutoff', 0, 'coll_time', [], 'coll_date', [],...
    'frame_start', 0, 'extended_header', 0, 'char_subitems', 0,...
    'int_subitems', 0, 'dbl_subitems', 0, 'item_size', 0, 'padding', 0);

fid = fopen(fileName, 'rb', 'ieee-le');

header.filetype = fread(fid, 1, 'int8');
header.items = fread(fid, 1, 'int16');
header.subitems = fread(fid, 1, 'int16');
header.numframes = fread(fid, 1, 'int32');
header.frequency = fread(fid, 1, 'float32');
header.user_comments = char(fread(fid, 60, 'uchar'))';
header.sys_comments = char(fread(fid, 60, 'uchar'))';
header.descrip_file = char(fread(fid, 30, 'uchar'))';
header.cutoff = fread(fid, 1, 'int16');
header.coll_time = char(fread(fid, 10, 'uchar'))';
header.coll_date = char(fread(fid, 10, 'uchar'))';
header.frame_start = fread(fid, 1, 'int32');
header.extended_header = fread(fid, 1, 'int16');
header.char_subitems = fread(fid, 1, 'int16');
header.int_subitems = fread(fid, 1, 'int16');
header.dbl_subitems = fread(fid, 1, 'int16');
header.item_size = fread(fid, 1, 'int16');
header.padding = char(fread(fid, 57, 'uchar'))';

if header.subitems == 0,	% NO floating subitems: check other data types
	if sum([header.int_subitems>0,header.dbl_subitems>0,header.char_subitems>0])>1,
		error('mixed subitems');
	end;
	if header.int_subitems > 0,  		% has integer subitems
		[data,count]=fread(fid,inf,'int16');
	    data = reshape(data,[header.int_subitems,count/header.int_subitems])';
	elseif header.dbl_subitems > 0,		% has floating point subitems
		[data,count]=fread(fid,inf,'float32');
	    data = reshape(data,[header.dbl_subitems,count/header.dbl_subitems])';
	elseif header.char_subitems > 0,	% has character subitems
		[data,count]=fread(fid,inf,'char');
	    data = reshape(data,[header.char_subitems,count/header.char_subitems])';
	end;
else, 						% floating subitems
	[data,count]=fread(fid,inf,'float32');
	n = header.items * header.subitems;
    data = reshape(data,[n,count/n])';
end;

fclose(fid);

if size(data,1) > header.numframes,		% match data length to header
	data = data(1:header.numframes,:);
elseif size(data,1) < header.numframes,
	header.numframes = size(data,1);
end;
