function m_out = ml_bracket(action, aux, aux2, aux3)
%ML_BRACKET  - MAVIS user labelling procedure
%
% Brackets the cursor location upon mouse release by finding nearest
% values matching specified criteria:
%
%  MAX  - boundary must be a local maximum >= % of trajectory maximum
%  MIN  - boundary must be a local minimum >= % of trajectory minimum
%  DIFF - boundary must differ from cursor value by >= % of trajectory range
%
% Examples:  
%	MAX, MAX brackets a valley
%	MAX, MIN delimits an opening gesture
%	DIFF, DIFF (low threshold) brackets a plateau

% 01/98 mkt

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
%	AUX		- internal state
%
% we need head, tail boundary selection modes, threshold, and dimension to test

	case 'CONFIG',
		state = aux;						% get internal state
		labelData = DoConfig(state.LABEL.USERDATA);
		if isempty(labelData),
			m_out = 0;				% report cancel
			return;
		end;
		state.LABEL.USERDATA = labelData;	% remember settings
		set(state.GUI, 'UserData', state);	% update internal state
		m_out = 1;							% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)
%
% bracket the cursor using specified criteria applied to clicked trajectory
% if it has less than preferred dimension use highest that it has

	case 'DOWN',
		global msecsToSamps sampsToMsecs;	% map time : samples
		state = aux;						% internal state
		t = aux2;								% index of clicked trajectory
		thresh = state.LABEL.USERDATA.THRESH;	% threshold
		modes = state.LABEL.USERDATA.MODES;		% [head tail]
		d = state.LABEL.USERDATA.DIM;			% preferred dimension
		
% get clicked trajectory data over the range of the current temporal selection
		sRate = state.DATA(t).SRATE;
		head = msecsToSamps(state.HEAD, sRate);
		tail = msecsToSamps(state.TAIL, sRate);
		cursor = msecsToSamps(state.CURSOR, sRate) - head + 1;
		nDims = size(state.DATA(t).SIGNAL,2);
		if nDims < d, d = nDims; end;
		data = state.DATA(t).SIGNAL(head:tail,d);	% working data vector

% find extrema sample indices, data : threshold ratios
		maxVal = max(data);
		minVal = min(data);
		spread = (maxVal - minVal);
		maxS = FindPeaks(data);
		rMax = (data - minVal) ./ spread;
		minS = FindPeaks(-data);
		rMin = 1 - rMax;
		rDiff = abs(data - data(cursor)) ./ spread;

% find the head
		errStr = 'Can''t bracket using specified criteria';
		switch modes(1),
			case 1,		% MAX
				i = intersect(maxS, find(rMax >= thresh));
				if ~isempty(i), i = i(find(i <= cursor)); end;
				if isempty(i), 	set(warndlg(errStr, 'ML_BRACKET'), 'WindowStyle', 'modal'); return; end;
				bh = i(end);
			case 2,		% MIN
				i = intersect(minS, find(rMin >= thresh));
				if ~isempty(i), i = i(find(i <= cursor)); end;
				if isempty(i), 	set(warndlg(errStr, 'ML_BRACKET'), 'WindowStyle', 'modal'); return; end;
				bh = i(end);
			case 3,		% DIFF
				i = intersect([1:cursor], find(rDiff >= thresh));
				if isempty(i), 	set(warndlg(errStr, 'ML_BRACKET'), 'WindowStyle', 'modal'); return; end;
				bh = i(end);
		end;
		bh = sampsToMsecs(bh+head-1, sRate);		% bracket head in msecs
		
% find the tail
		switch modes(2),
			case 1,		% MAX
				i = intersect(maxS, find(rMax >= thresh));
				if ~isempty(i), i = i(find(i >= cursor)); end;
				if isempty(i), 	set(warndlg(errStr, 'ML_BRACKET'), 'WindowStyle', 'modal'); return; end;
				bt = i(1);
			case 2,		% MIN
				i = intersect(minS, find(rMin >= thresh));
				if ~isempty(i), i = i(find(i >= cursor)); end;
				if isempty(i), 	set(warndlg(errStr, 'ML_BRACKET'), 'WindowStyle', 'modal'); return; end;
				bt = i(1);
			case 3,		% DIFF
				i = intersect([cursor:tail], find(rDiff >= thresh));
				if isempty(i), 	set(warndlg(errStr, 'ML_BRACKET'), 'WindowStyle', 'modal'); return; end;
				bt = i(1);
		end;
		bt = sampsToMsecs(bt+head-1, sRate);		% bracket tail in msecs
		
% make the labels
		saveAnnotate = state.LABEL.ANNOTATE;		% get current annotation status
		state.LABEL.ANNOTATE = 0;					% force it off for quiet label creation
		set(state.GUI, 'UserData', state);

		trajName = state.DATA(t).NAME;				% build name 
		if nDims > 1,
			xyz = 'XYZ';
			trajName = [trajName '_' xyz(d)]; 
		end;
		modeNames = {'MAX' 'MIN' 'DIFF'};

		label = struct('NAME', 'head', ...
					'OFFSET', bh, ...				
					'NOTE', [trajName ' head (' char(modeNames(modes(1))) ', thresh = ' num2str(thresh) ')'], ...					
					'VALUE', []);
		m_label('MAKE', label);
		label = struct('NAME', 'tail', ...
					'OFFSET', bt, ...				
					'NOTE', [trajName ' tail (' char(modeNames(modes(2))) ', thresh = ' num2str(thresh) ')'], ...					
					'VALUE', []);
		m_label('MAKE', label);
		
		state = get(state.GUI, 'UserData');			% retrieve internal state
		state.LABEL.ANNOTATE = saveAnnotate;		% restore annotation status
		set(state.GUI, 'UserData', state);


%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label
%
% call the default plotting routine

	case 'PLOT',
		m_label('PLOT', aux, aux2, aux3);
		

%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_BRACKET:  unrecognized action (', action, ')']);
	
end;

%=============================================================================
% DOCONFIG  - config handler
%
%   returns non-empty labelData on OK, empty result on cancel

function labelData = DoConfig(labelData)

figPos = get(0, 'ScreenSize');
width = 450;
height = 250;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

idString = 'ML_BRACKET';
if isempty(labelData) | ~strcmp(labelData.SOURCE, idString),		% if not previously configured
	labelData = struct('SOURCE', idString, ...
						'MODES', [1 1], ...
						'THRESH', .75, ...
						'DIM', 1);
end;

cfg = dialog('Name', idString, ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

% about
uicontrol(cfg, ...
	'Position', [10 height-120 width-20 110], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 height-45 width-26 30], ...
	'Style', 'text', ...
	'String', 'Brackets the cursor location upon mouse release by finding nearest values matching specified criteria:', ...
	'HorizontalAlignment', 'left');
uicontrol(cfg, ...
	'Position', [17 height-70 35 20], ...
	'Style', 'text', ...
	'String', 'MAX', ...
	'HorizontalAlignment', 'left');
uicontrol(cfg, ...
	'Position', [54 height-70 width-70 20], ...
	'Style', 'text', ...
	'String', '- boundary must be a local maximum >= threshold % of trajectory max', ...
	'HorizontalAlignment', 'left');
uicontrol(cfg, ...
	'Position', [17 height-90 35 20], ...
	'Style', 'text', ...
	'String', 'MIN', ...
	'HorizontalAlignment', 'left');
uicontrol(cfg, ...
	'Position', [54 height-90 width-70 20], ...
	'Style', 'text', ...
	'String', '- boundary must be a local minimum >= threshold % of trajectory min', ...
	'HorizontalAlignment', 'left');
uicontrol(cfg, ...
	'Position', [17 height-110 35 20], ...
	'Style', 'text', ...
	'String', 'DIFF', ...
	'HorizontalAlignment', 'left');
uicontrol(cfg, ...
	'Position', [54 height-110 width-70 20], ...
	'Style', 'text', ...
	'String', '- must differ from cursor val by >= threshold % of trajectory range', ...
	'HorizontalAlignment', 'left');

% head boundary
uicontrol(cfg, ...
	'Position', [15 90 45 20], ...
	'Style', 'text', ...
	'String', 'Head:', ...
	'HorizontalAlignment', as);
head = uicontrol(cfg, ...
	'Position',[65 90 60 20], ...
	'HorizontalAlignment', 'left', ...
	'String','MAX|MIN|DIFF', ...
	'Style','popupmenu', ...
	'Value', labelData.MODES(1));
	
% tail boundary
uicontrol(cfg, ...
	'Position', [135 90 45 20], ...
	'Style', 'text', ...
	'String', 'Tail:', ...
	'HorizontalAlignment', as);
tail = uicontrol(cfg, ...
	'Position',[185 90 60 20], ...
	'HorizontalAlignment', 'left', ...
	'String','MAX|MIN|DIFF', ...
	'Style','popupmenu', ...
	'Value', labelData.MODES(1));
	
% threshold
uicontrol(cfg, ...
	'Position', [270 90 60 17], ...
	'HorizontalAlignment', as, ...
	'String','Threshold:', ...
	'Style','text');
thresh = uicontrol(cfg, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'String', num2str(labelData.THRESH), ...
	'Position', [335 90 50 20]);

% select dimension
uicontrol(cfg, ...
	'Position', [90 55 115 17], ...
	'HorizontalAlignment', as, ...
	'String','Apply to dimension:', ...
	'Style','text');
xRB = uicontrol(cfg, ...
	'Position', [210 55 40 20], ...
	'String','X', ...
	'Style','radiobutton', ...
	'Value', (labelData.DIM==1), ...
	'Callback', 'set(gcbo,''Value'',1);set(get(gcbo,''UserData''),''Value'',~get(gcbo,''Value''));');
yRB = uicontrol(cfg, ...
	'Position', [255 55 40 20], ...
	'String','Y', ...
	'Style','radiobutton', ...
	'Value', (labelData.DIM==2), ...
	'Callback', 'set(gcbo,''Value'',1);set(get(gcbo,''UserData''),''Value'',~get(gcbo,''Value''));');
zRB = uicontrol(cfg, ...
	'Position', [300 55 40 20], ...
	'String','Z', ...
	'Style','radiobutton', ...
	'Value', (labelData.DIM==3), ...
	'Callback', 'set(gcbo,''Value'',1);set(get(gcbo,''UserData''),''Value'',~get(gcbo,''Value''));');
set(xRB, 'UserData', [yRB zRB]);
set(yRB, 'UserData', [xRB zRB]);
set(zRB, 'UserData', [xRB yRB]);

% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
if get(cfg, 'UserData'),
	labelData.MODES = [get(head, 'Value') get(tail, 'Value')];
	labelData.THRESH = str2num(get(thresh, 'String'));
	labelData.DIM = find([get(xRB, 'Value') get(yRB, 'Value') get(zRB, 'Value')]);
else,
	labelData = [];
end;
delete(cfg);


%=============================================================================
% FINDPEAKS  - find indices of vector maxima (from SPBROWSER)

function i = FindPeaks(y)

switch length(y),
	case 0, i = [];
	case 1, i = 1;
	otherwise,
		dy = diff(y);
		i = find(([dy;0]<0) & ([0;dy]>=0));
		if y(end-1) <= y(end), i = [i;length(y)]; end;
end;
