function m_out = ml_findextrema(action, aux, aux2, aux3)
%ML_FINDEXTREMA  - MAVIS user labelling procedure
%
% Labels the velocity peaks of the selected trajectory
% exceeding user-specified threshold.  NOTE field set
% to labelled peak percentage of maximum velocity.
% Optionally plots trajectory and superimposed velocity
% in a new zoomable window.

% 01/98 mkt

%	branch by action...

switch action,
		
%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
%	AUX		- internal state
%
% set labelling threshold, get plotting status

	case 'CONFIG',
		state = aux;						% get internal state
		labelData = DoConfig(state.LABEL.USERDATA);
		if isempty(labelData) | labelData.THRESH<=0 | labelData.THRESH>1,
			m_out = 0;				% report cancel
			return;
		end;
		state.LABEL.USERDATA = labelData;
		set(state.GUI, 'UserData', state);	% update internal state
		m_out = 1;							% report OK
		return;
		
		
%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)
%
% label velocity peaks exceeding threshold

	case 'DOWN',
		global msecsToSamps sampsToMsecs;	% map time : samples
		state = aux;						% internal state
		t = aux2;							% index of clicked trajectory
		thresh = state.LABEL.USERDATA.THRESH;		% retrieve settings
		doPlot = state.LABEL.USERDATA.PLOT;
		
% get clicked trajectory data over the range of the current temporal selection
		sRate = state.DATA(t).SRATE;
		head = msecsToSamps(state.HEAD, sRate);
		tail = msecsToSamps(state.TAIL, sRate);
		data = state.DATA(t).SIGNAL(head:tail,:);

% compute trajectory velocities (central difference)
		vel = zeros(size(data));
		vel(2:end-1,:) = (data(3:end,:) - data(1:end-2,:)) ./ 2;
		vel(1,:) = vel(2,:);				% pad endpoints
		vel(end,:) = vel(end-1,:);
		vel = sqrt(sum(vel.^2,2));			% velocity magnitude

% find peaks
		dv = diff(vel);
		i = find(([dv;0]<0) & ([0;dv]>=0));			% velocity peak sample indices
		rv = vel./max(vel);							% velocity % of maxVel
		peakS = intersect(i, find(rv >= thresh));	% peak samples
		peakT = sampsToMsecs(peakS+head-1, sRate);	% peak msecs

% label
		saveAnnotate = state.LABEL.ANNOTATE;		% get current annotation status
		state.LABEL.ANNOTATE = 0;					% force it off for quiet label creation
		set(state.GUI, 'UserData', state);

		label = struct('NAME', '', ...
					'OFFSET', [], ...				
					'NOTE', [], ...					
					'VALUE', []);
		for i = 1 : length(peakS),
			label.OFFSET = peakT(i);
			label.NOTE = num2str(rv(peakS(i)));
			m_label('MAKE', label);
		end;
		
		state = get(state.GUI, 'UserData');			% retrieve internal state
		state.LABEL.ANNOTATE = saveAnnotate;		% restore annotation status
		set(state.GUI, 'UserData', state);
		if ~doPlot, return; end;

% means center
		[nSamps, nDims] = size(data);
		mins = state.DATA(t).SCALING.RANGE(:,1)';
		spreads = (state.DATA(t).SCALING.RANGE(:,2)' - mins) / 2;
		spread = max(spreads);
		spread = spread + spread * 0.2;		% squish slightly to fit within axis
		data = data(1:nSamps,:) - repmat(mins+spreads,nSamps,1);
		rv = rv * 2*spread - spread;
		
% plot
		figure('Name', state.DATA(t).NAME);
		plot([data rv]);									% data, velocity
		set(gca, 'Xlim', [1 nSamps], 'Ylim', [-spread spread], 'Box', 'on');
		line('Xdata', [1 nSamps], 'Ydata', [0 0], ...
				'Color', [.5 .5 .5], 'LineStyle', ':');		% midline
		for i = 1 : length(peakS),							% labels
			line('XData', [peakS(i) peakS(i)], 'YData', [-spread spread]);
		end;
		names = {'X','Y','Z','Velocity'};
		if nDims < 3, names(3) = []; end;
		legend(names{:});
		xlabel('samps');
		zoom on;
		

%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label
%
% call the default plotting routine

	case 'PLOT',
		m_label('PLOT', aux, aux2, aux3);
		

%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_VELPEAKS:  unrecognized action (', action, ')']);
	
end;


%=============================================================================
% DOCONFIG  - config handler
%
%   returns non-empty labelData on OK, empty result on cancel

function labelData = DoConfig(labelData)

figPos = get(0, 'ScreenSize');
width = 260;
height = 260;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

idString = 'ML_VELPEAKS';
if isempty(labelData) | ~strcmp(labelData.SOURCE, idString),		% if not previously configured
	labelData = struct('SOURCE', idString, ...
						'THRESH', .75, ...
						'PLOT', 1);
end;

cfg = dialog('Name', idString, ...
	'Position', figPos, ...
	'ButtonDownFcn', '', ...
	'UserData', 0);

% about
blurb = ['Labels the velocity peaks of the selected trajectory ' ...
		'exceeding user-specified threshold.  NOTE field set ' ...
		'to labelled peak percentage of maximum velocity.  ' ...
		'Optionally plots trajectory and superimposed velocity ' ...
		'in a new zoomable window.'];

uicontrol(cfg, ...
	'Position', [10 height-120 width-20 110], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 height-117 width-26 104], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', blurb);

% editable text field and label
uicontrol(cfg, ...
	'Position', [5 100 175 17], ...
	'HorizontalAlignment', as, ...
	'String','Threshold (% of max velocity):', ...
	'Style','text');
thresh = uicontrol(cfg, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'String', num2str(labelData.THRESH), ...
	'Position', [185 100 50 20]);

% checkbox
doPlot = uicontrol(cfg, ...
	'Style', 'checkbox', ...
	'HorizontalAlignment', 'left', ...
	'Position', [30, 65 210 20], ...
	'String', 'Plot results in separate figure', ...
	'Value', labelData.PLOT);

% OK, cancel buttons
uicontrol(cfg, ...		% buttons
	'Position',[width/2-70 15 60 25], ...
	'String','OK', ...
	'Callback','set(gcbf,''UserData'',1);uiresume');
uicontrol(cfg, ...
	'Position',[width/2+10 15 60 25], ...
	'String','Cancel', ...
	'Callback','uiresume');

% wait for input
uiwait(cfg);
if get(cfg, 'UserData'),
	labelData.THRESH = str2num(get(thresh, 'String'));
	labelData.PLOT = get(doPlot, 'Value');
else,
	labelData = [];
end;
delete(cfg);
