function m_out = ml_velzero(action, aux, aux2, aux3)
%ML_VELZERO  - MAVIS user labelling procedure
%
% Louis Goldstein
%
% 06/99
%
% Labels the crossings from from a non-zero
% (tangential) velocity to zero velocity and vice versa.
%
% "zero" velocity is defined by a user-specified threhold
%
% Optionally plots trajectory and superimposed velocity
% in a new zoomable window.
%
% based on ml_template.m, ml_velpeaks.m by mkt

% mkt 08/99 added persistent configuration, selectable component
% mkt 12/00 fix velocity magnitude calculation

%	branch by action...

switch action,

%-----------------------------------------------------------------------------
% CLOSE:  close mavis as well as config window

	case 'CLOSE',
		labelData = get(gcbf, 'userdata');
		if ishandle(labelData.GUI), m_gui('CLOSE'); end;
		delete(gcbf);


%-----------------------------------------------------------------------------
% CONFIG:  handle configuration
%
%	AUX		- internal state
%
% set labelling threshold, get plotting status

	case 'CONFIG',
		DoConfig(aux);
		m_out = 1;							% report OK


%-----------------------------------------------------------------------------
% DOWN:  handle mouseDown
%
%	AUX		- internal state
%	AUX2	- index of clicked trajectory (into state.DATA)
%
% label velocity peaks exceeding threshold

	case 'DOWN',
		global msecsToSamps sampsToMsecs;		% map time : samples
		state = aux;							% internal state
		t = aux2;								% index of clicked trajectory
		thresh = state.LABEL.USERDATA.THRESH;	% retrieve settings
		doPlot = state.LABEL.USERDATA.PLOT;
		if isempty(strmatch('VELSRC', char(fieldnames(state.LABEL.USERDATA)), 'exact'));	
			velSrc = 3;		%  backwards compatibility (sigh)
		else,
			velSrc = state.LABEL.USERDATA.VELSRC;
		end;

% get clicked trajectory data over the range of the current temporal selection
		sRate = state.DATA(t).SRATE;
		head = msecsToSamps(state.HEAD, sRate);
		tail = msecsToSamps(state.TAIL, sRate);
		data = state.DATA(t).SIGNAL(head:tail,:);

% compute trajectory velocities (central difference)
		vel = zeros(size(data));
		vel(2:end-1,:) = (data(3:end,:) - data(1:end-2,:)) ./ 2;
		vel(1,:) = vel(2,:);				% pad endpoints
		vel(end,:) = vel(end-1,:);
		if velSrc == 3,
			vel = sqrt(sum(vel.^2,2));			% velocity magnitude
		else,
			vel = abs(vel(:,velSrc));			% abs component velocity
		end;

% find zeroes
		threshS = thresh/sRate;					% convert thresh to mm/sample
		i1 = find(([vel;vel(end)]<threshS) & ([vel(1);vel]>=threshS));	% repeat initial or final
		i2 = find(([vel;vel(end)]>=threshS) & ([vel(1);vel]<threshS));	% values as padding
		peakS = setxor([i1;i2],[2;length(data)]); 	% get rid of bogus zero due to padding
		peakT = sampsToMsecs(peakS+head-1, sRate);	% peak msecs

% label
		saveAnnotate = state.LABEL.ANNOTATE;		% get current annotation status
		state.LABEL.ANNOTATE = 0;					% force it off for quiet label creation
		set(state.GUI, 'UserData', state);

		label = struct('NAME', '', ...
					'OFFSET', [], ...
					'NOTE', [], ...
					'VALUE', []);
		for i = 1 : length(peakS),
			label.OFFSET = peakT(i);
			label.NOTE = [' VELZERO: thresh = ' num2str(thresh) ' mm/sec'];
			m_label('MAKE', label);
		end;

		state = get(state.GUI, 'UserData');			% retrieve internal state
		state.LABEL.ANNOTATE = saveAnnotate;		% restore annotation status
		set(state.GUI, 'UserData', state);
		if ~doPlot, return; end;

% means center
		[nSamps, nDims] = size(data);
		mins = state.DATA(t).SCALING.RANGE(:,1)';
		spreads = (state.DATA(t).SCALING.RANGE(:,2)' - mins) / 2;
		if velSrc < 3,
			data = data(:,velSrc);
			mins = mins(velSrc);
			spreads = spreads(velSrc);
		end;
		spread = max(spreads);
		spread = spread + spread * 0.2;		% squish slightly to fit within axis
		data = data(1:nSamps,:) - repmat(mins+spreads,nSamps,1);
		vel_scale = vel * spread*2 - spread;
		thresh_line = threshS * spread*2 - spread;
		max_line = max(vel) * spread*2 - spread;

% plot
		h = findobj('tag', 'VZPLOT');
		if ishandle(h),
			figure(h);
		else,
			figure('tag','VZPLOT','name', state.DATA(t).NAME);
		end;
		plot([data vel_scale]);
		switch velSrc,
			case 1, legend('X', 'Vel');
			case 2, legend('Y', 'Vel');
			case 3, legend('X', 'Y', 'Vel');
		end;
				% data, velocity
		set(gca, 'Xlim', [1 nSamps], 'Ylim', [-spread spread], 'Box', 'on');
		line('Xdata', [1 nSamps], 'Ydata', [thresh_line thresh_line], ...
				'Color', [.5 .5 .5], 'LineStyle', ':');	% threshold
		line('Xdata', [1 nSamps], 'Ydata', [max_line max_line], ...
				'Color', [.5 .5 .5], 'LineStyle', '--');	% max
		for i = 1 : length(peakS),	% labels
			line('XData', [peakS(i) peakS(i)], 'YData', [-spread spread]);
		end;
		title ([' Threshold = ' num2str(thresh) ...
				'; Max Velocity in window = ' num2str(max(vel)*sRate)]);
		set(gca,'yticklabel',num2str(round((str2num(get(gca,'yticklabel'))+spread)*sRate/(spread*2))));
		ylabel('mm/sec');
		xlabel('samps');
		zoom on;


%-----------------------------------------------------------------------------
% PLOT:  plot label into the current axes
%	AUX		- x offset (in trajectory sample units) of label
%	AUX2	- yLim of current axis
%	AUX3	- label
%
% call the default plotting routine

	case 'PLOT',
		m_label('PLOT', aux, aux2, aux3);


%-----------------------------------------------------------------------------
% UPDATE:  update configuration

	case 'UPDATE',
		labelData = get(gcbf, 'userdata');

% twiddle radio buttons
		if aux < 4,
			if get(gcbo, 'value'), 
				set(labelData.RBH(find(aux~=[1:3])), 'value', 0); 
			else, 
				set(gcbo, 'value', 1); 
			end;
		end;
		
% update stored configuration
		n = str2num(get(labelData.THRH, 'string'));
		if n <= 0,
			set(labelData.THRH, 'string', num2str(labelData.THRESH));
		else,
			labelData.THRESH = n;
		end;
		labelData.PLOT = get(labelData.PLOTH, 'value');
		n = get(labelData.RBH, 'value')';
		labelData.VELSRC = find([n{:}]);
		state = get(labelData.GUI, 'userdata');
		state.LABEL.USERDATA = labelData;
		set(state.GUI, 'UserData', state);	% update internal state


%-----------------------------------------------------------------------------
% error

	otherwise,
		error(['ML_VELZEROS:  unrecognized action (', action, ')']);

end;


%=============================================================================
% DOCONFIG  - config handler
%
%   returns handle to cfg dialog

function cfg = DoConfig(state)

labelData = state.LABEL.USERDATA;

figPos = get(0, 'ScreenSize');
width = 260;
height = 220;
figPos = [figPos(1)+(figPos(3)-width)/2, figPos(2)+(figPos(4)-height)/2, width, height];
if strncmp(computer, 'SGI', 3), as = 'left'; else, as = 'right'; end;	% SGI bug:  doesn't like right alignment

idString = 'ML_VELZEROS';
if isempty(labelData) | ~strcmp(labelData.SOURCE, idString),				% if not previously configured
	labelData = struct('SOURCE', idString, ...
						'THRESH', 100, ...			% threshold
						'PLOT', 1, ...				% separate plot
						'VELSRC', 3);				% velocity source
elseif isempty(strmatch('VELSRC', char(fieldnames(labelData)), 'exact'));	
	labelData.VELSRC = 3;		%  backwards compatibility (sigh)
end;
labelData.THRH = [];			% control handles
labelData.PLOTH = [];
labelData.RBH = zeros(1,3);
labelData.GUI = state.GUI;

cfg = findobj('tag', 'VZCONFIG');
if ishandle(cfg),		% if dialog already exists pop it forward
	figure(cfg);
else,					% else create it
	cfg = dialog('Name', idString, ...
		'Position', figPos, ...
		'ButtonDownFcn', '', ...
		'CloseRequestFcn', 'ml_velzero CLOSE;', ...
		'windowstyle', 'normal', ...
		'tag', 'VZCONFIG', ...
		'visible', 'off', ...
		'menubar', 'none', ...
		'UserData', 0);
end;

% about
blurb = ['Labels the crossings from from a non-zero' ...
		 ' (tangential) velocity to zero velocity and vice versa.' ...
		 ' "zero" velocity is defined by a user-specified threhold.' ...
		 ' Optionally plots trajectory and superimposed velocity' ...
		 ' in a new zoomable window.'];
uicontrol(cfg, ...
	'Position', [10 117 width-20 90], ...
	'Style', 'frame');
uicontrol(cfg, ...
	'Position', [13 120 width-26 84], ...
	'Style', 'text', ...
	'HorizontalAlignment', 'left', ...
	'String', blurb);

% editable text field and label
uicontrol(cfg, ...
	'Position', [5 80 185 17], ...
	'HorizontalAlignment', as, ...
	'String','Zero Velocity Threshold (mm/sec):', ...
	'Style','text');
labelData.THRH = uicontrol(cfg, ...
	'Style', 'edit', ...
	'HorizontalAlignment', 'left', ...
	'String', num2str(labelData.THRESH), ...
	'Position', [195 80 50 20], ...
	'callback', 'ml_velzero(''UPDATE'',4);');

% checkbox
labelData.PLOTH = uicontrol(cfg, ...
	'Style', 'checkbox', ...
	'HorizontalAlignment', 'left', ...
	'Position', [40, 50 210 20], ...
	'String', 'Plot results in separate figure', ...
	'Value', labelData.PLOT, ...
	'callback', 'ml_velzero(''UPDATE'',5);');

% source
uicontrol(cfg, ...
	'Position', [5 20 100 17], ...
	'HorizontalAlignment', as, ...
	'String','Velocity source:', ...
	'Style','text');
labelData.RBH(1) = uicontrol(cfg, ...		% X
	'Style', 'Radiobutton', ...
	'HorizontalAlignment', 'left', ...
	'String', 'X', ...
	'Value', (labelData.VELSRC == 1), ...
	'Position', [110 20 30 20], ...
	'callback', 'ml_velzero(''UPDATE'',1);');
labelData.RBH(2) = uicontrol(cfg, ...		% Y
	'Style', 'Radiobutton', ...
	'HorizontalAlignment', 'left', ...
	'String', 'Y', ...
	'Value', (labelData.VELSRC == 2), ...
	'Position', [145 20 30 20], ...
	'callback', 'ml_velzero(''UPDATE'',2);');
labelData.RBH(3) = uicontrol(cfg, ...		% both
	'Style', 'Radiobutton', ...
	'HorizontalAlignment', 'left', ...
	'String', 'both', ...
	'Value', (labelData.VELSRC == 3), ...
	'Position', [180 20 60 20], ...
	'callback', 'ml_velzero(''UPDATE'',3);');

% save labelData as cfg window userdata

set(cfg, 'userdata', labelData, 'visible', 'on');
state.LABEL.USERDATA = labelData;
set(state.GUI, 'UserData', state);	% update internal state
